@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
@@ -147,14 +148,6 @@ MagickExport MagickBooleanType AcquireUniqueFilename(char *path)
 %
 */
 
-static inline MagickSizeType MagickMin(const MagickSizeType x,
-  const MagickSizeType y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
   char *destination)
 {
@@ -210,9 +203,8 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
       return(MagickFalse);
     }
   quantum=(size_t) MagickMaxBufferExtent;
-  if ((fstat(source_file,&attributes) == 0) && (attributes.st_size != 0))
-    quantum=(size_t) MagickMin((size_t) attributes.st_size,
-      MagickMaxBufferExtent);
+  if ((fstat(source_file,&attributes) == 0) && (attributes.st_size > 0))
+    quantum=(size_t) MagickMin(attributes.st_size,MagickMaxBufferExtent);
   buffer=(unsigned char *) AcquireQuantumMemory(quantum,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     {
@@ -353,7 +345,7 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
   assert(source != (char *) NULL);
   assert(length != (size_t *) NULL);
   *length=0;
-  decode=(unsigned char *) AcquireQuantumMemory(strlen(source)/4+4,
+  decode=(unsigned char *) AcquireQuantumMemory((strlen(source)+3)/4,
     3*sizeof(*decode));
   if (decode == (unsigned char *) NULL)
     return((unsigned char *) NULL);
@@ -727,7 +719,6 @@ MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
   char ***arguments)
 {
   char
-    *directory,
     home_directory[MaxTextExtent],
     **vector;
 
@@ -814,8 +805,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
         GetPathComponent(option,SubimagePath,subimage);
         ExpandFilename(path);
         if (*home_directory == '\0')
-          directory=getcwd(home_directory,MaxTextExtent-1);
-        (void) directory;
+          getcwd_utf8(home_directory,MaxTextExtent-1);
         filelist=ListFiles(*path == '\0' ? home_directory : path,filename,
           &number_files);
       }
@@ -865,7 +855,12 @@ MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
     vector=(char **) ResizeQuantumMemory(vector,(size_t) *number_arguments+
       count+number_files+1,sizeof(*vector));
     if (vector == (char **) NULL)
-      return(MagickFalse);
+      {
+        for (j=0; j < (ssize_t) number_files; j++)
+          filelist[j]=DestroyString(filelist[j]);
+        filelist=(char **) RelinquishMagickMemory(filelist);
+        return(MagickFalse);
+      }
     for (j=0; j < (ssize_t) number_files; j++)
     {
       option=filelist[j];
@@ -1160,6 +1155,7 @@ MagickExport MagickBooleanType GetPathAttributes(const char *path,
       errno=EINVAL;
       return(MagickFalse);
     }
+  (void) ResetMagickMemory(attributes,0,sizeof(struct stat));
   status=stat_utf8(path,(struct stat *) attributes) == 0 ? MagickTrue :
     MagickFalse;
   return(status);
@@ -1442,6 +1438,8 @@ MagickExport MagickBooleanType IsPathAccessible(const char *path)
 
   if ((path == (const char *) NULL) || (*path == '\0'))
     return(MagickFalse);
+  if (strcmp(path,"-") == 0)
+    return(MagickTrue);
   status=GetPathAttributes(path,&attributes);
   if (status == MagickFalse)
     return(status);
@@ -1768,7 +1766,7 @@ MagickExport size_t MultilineCensus(const char *label)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S h r e a d F i l e                                                       %
+%   S h r e d F i l e                                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1819,7 +1817,11 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
       */
       status=remove_utf8(path);
       if (status == -1)
-        return(MagickFalse);
+        {
+          (void) LogMagickEvent(ExceptionEvent,GetMagickModule(),
+            "Failed to remove: %s",path);
+          return(MagickFalse);
+        }
       return(MagickTrue);
     }
   file=open_utf8(path,O_WRONLY | O_EXCL | O_BINARY,S_MODE);
@@ -1829,15 +1831,17 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
         Don't shred the file, just remove it.
       */
       status=remove_utf8(path);
+      if (status == -1)
+        (void) LogMagickEvent(ExceptionEvent,GetMagickModule(),
+          "Failed to remove: %s",path);
       return(MagickFalse);
     }
   /*
     Shred the file.
   */
   quantum=(size_t) MagickMaxBufferExtent;
-  if ((fstat(file,&file_stats) == 0) && (file_stats.st_size != 0))
-    quantum=(size_t) MagickMin((MagickSizeType) file_stats.st_size,
-      MagickMaxBufferExtent);
+  if ((fstat(file,&file_stats) == 0) && (file_stats.st_size > 0))
+    quantum=(size_t) MagickMin(file_stats.st_size,MagickMaxBufferExtent);
   length=(MagickSizeType) file_stats.st_size;
   for (i=0; i < (ssize_t) StringToInteger(passes); i++)
   {
